Imports Microsoft.DirectX.DirectInput

Public Class MainForm
    Public dataOutput(16) As Byte       ' output data to transmit
    Private _device As Device           ' joystick device
    Private _jstate As JoystickState    ' state of joystick axes

    ''' <summary>
    ''' Called automatically at startup of the programm, search all available COM-Ports
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.Text = My.Application.Info.Title & " " & My.Application.Info.Version.ToString()
        ' scan COM-Ports
        For i As Integer = 0 To My.Computer.Ports.SerialPortNames.Count - 1
            ComboBox1.Items.Add(My.Computer.Ports.SerialPortNames(i))
        Next

        Me.initJoystick()
        Me.setScrollbarRanges()
    End Sub


    ''' <summary>
    ''' Applies min/max values of 1st scrollbar slider (set in GUI) to all sliders
    ''' </summary>
    ''' <remarks></remarks>
    Sub setScrollbarRanges()
        Dim max As Integer = Me.VScrollBar1.Maximum
        Dim min As Integer = Me.VScrollBar1.Minimum

        Me.VScrollBar2.Maximum = max
        Me.VScrollBar3.Maximum = max
        Me.VScrollBar4.Maximum = max
        Me.VScrollBar5.Maximum = max
        Me.VScrollBar6.Maximum = max
        Me.VScrollBar7.Maximum = max
        Me.VScrollBar8.Maximum = max

        Me.VScrollBar2.Minimum = min
        Me.VScrollBar3.Minimum = min
        Me.VScrollBar4.Minimum = min
        Me.VScrollBar5.Minimum = min
        Me.VScrollBar6.Minimum = min
        Me.VScrollBar7.Minimum = min
        Me.VScrollBar8.Minimum = min

        VScrollBarJoyX.Maximum = max
        VScrollBarJoyY.Maximum = max
        VScrollBarJoyZ.Maximum = max
        VScrollBarJoyRx.Maximum = max
        VScrollBarJoyRy.Maximum = max
        VScrollBarJoyRz.Maximum = max
        VScrollBarJoyEx1.Maximum = max
        VScrollBarJoyEx2.Maximum = max

        VScrollBarJoyX.Minimum = min
        VScrollBarJoyY.Minimum = min
        VScrollBarJoyZ.Minimum = min
        VScrollBarJoyRx.Minimum = min
        VScrollBarJoyRy.Minimum = min
        VScrollBarJoyRz.Minimum = min
        VScrollBarJoyEx2.Minimum = min
        VScrollBarJoyEx2.Minimum = min

        Dim max_min2 As Integer = (max - min) / 2
        Me.VScrollBar1_Offset.Maximum = max_min2
        Me.VScrollBar2_Offset.Maximum = max_min2
        Me.VScrollBar3_Offset.Maximum = max_min2
        Me.VScrollBar4_Offset.Maximum = max_min2
        Me.VScrollBar5_Offset.Maximum = max_min2
        Me.VScrollBar6_Offset.Maximum = max_min2
        Me.VScrollBar7_Offset.Maximum = max_min2
        Me.VScrollBar8_Offset.Maximum = max_min2

        Me.VScrollBar1_Offset.Minimum = -max_min2
        Me.VScrollBar2_Offset.Minimum = -max_min2
        Me.VScrollBar3_Offset.Minimum = -max_min2
        Me.VScrollBar4_Offset.Minimum = -max_min2
        Me.VScrollBar5_Offset.Minimum = -max_min2
        Me.VScrollBar6_Offset.Minimum = -max_min2
        Me.VScrollBar7_Offset.Minimum = -max_min2
        Me.VScrollBar8_Offset.Minimum = -max_min2

        Me.VScrollBar1_Offset.Value = 0
        Me.VScrollBar2_Offset.Value = 0
        Me.VScrollBar3_Offset.Value = 0
        Me.VScrollBar4_Offset.Value = 0
        Me.VScrollBar5_Offset.Value = 0
        Me.VScrollBar6_Offset.Value = 0
        Me.VScrollBar7_Offset.Value = 0
        Me.VScrollBar8_Offset.Value = 0

        ComboBoxJoyX.SelectedIndex = 0
        ComboBoxJoyY.SelectedIndex = 1
        ComboBoxJoyZ.SelectedIndex = 2
        ComboBoxJoyRx.SelectedIndex = 3
        ComboBoxJoyRy.SelectedIndex = 4
        ComboBoxJoyRz.SelectedIndex = 5
        ComboBoxJoyE1.SelectedIndex = 6
        ComboBoxJoyE2.SelectedIndex = 7
    End Sub

    ''' <summary>
    ''' Searches joystick and sets range for axes
    ''' </summary>
    ''' <remarks></remarks>
    Sub initJoystick()
        Dim gameControllerList As DeviceList
        gameControllerList = Manager.GetDevices(DeviceClass.GameControl, EnumDevicesFlags.AttachedOnly)
        If (gameControllerList.Count > 0) Then
            Dim deviceInstance As DeviceInstance
            For Each deviceInstance In gameControllerList
                _device = New Device(deviceInstance.InstanceGuid)
                _device.SetCooperativeLevel(Me, CooperativeLevelFlags.Background Or CooperativeLevelFlags.NonExclusive)
                For Each d As DeviceObjectInstance In _device.Objects
                    If 0 <> (d.ObjectId And CInt(DeviceObjectTypeFlags.Axis)) Then
                        ' Set the range for the axis.
                        _device.Properties.SetRange(ParameterHow.ById, d.ObjectId, New InputRange(Me.VScrollBar1.Minimum, Me.VScrollBar1.Maximum))
                    End If
                Next d
            Next
        End If
        If Not _device Is Nothing Then
            _device.SetDataFormat(DeviceDataFormat.Joystick)
            MsgBox("Using Joystick : """ & _device.DeviceInformation.InstanceName & """")
        End If
    End Sub

    ''' <summary>
    ''' reads status of joystick
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub pollJoystick()
        If Not _device Is Nothing Then
            Try
                _device.Poll()
            Catch inputex As InputException
                If TypeOf inputex Is NotAcquiredException Or TypeOf inputex Is InputLostException Then
                    Try
                        _device.Acquire()
                    Catch
                        MsgBox("Error 1")
                        Return
                    End Try
                End If
            End Try

            _jstate = _device.CurrentJoystickState
        End If
    End Sub

    ''' <summary>
    ''' Reads joystick axes and moves the sliders on GUI
    ''' </summary>
    ''' <remarks></remarks>
    Sub readJoystickInput()
        Try
            pollJoystick()
            Me.VScrollBarJoyX.Value = _jstate.X
            Me.VScrollBarJoyY.Value = _jstate.Y
            Me.VScrollBarJoyZ.Value = _jstate.Z
            Me.VScrollBarJoyRx.Value = _jstate.Rx
            Me.VScrollBarJoyRy.Value = _jstate.Ry
            Me.VScrollBarJoyRz.Value = _jstate.Rz
            Me.VScrollBarJoyEx1.Value = _jstate.GetSlider(0)
            Me.VScrollBarJoyEx2.Value = _jstate.GetSlider(1)
            Me.CheckBoxButton1.Checked = _jstate.GetButtons(0)
            Me.CheckBoxButton2.Checked = _jstate.GetButtons(1)
            Me.CheckBoxButton3.Checked = _jstate.GetButtons(2)
            Me.CheckBoxButton4.Checked = _jstate.GetButtons(3)
            Me.CheckBoxButton5.Checked = _jstate.GetButtons(4)
            Me.CheckBoxButton6.Checked = _jstate.GetButtons(5)
            Me.CheckBoxButton7.Checked = _jstate.GetButtons(6)
            Me.CheckBoxButton8.Checked = _jstate.GetButtons(7)
        Catch ex As Exception
            cbJoystickActive.Checked = False
            MsgBox("Joystick Failure")
        End Try

    End Sub

    Sub writeMixedJoystickAxes2PPMChannel()
        setScrollbarValue(ComboBoxJoyX.SelectedIndex + 1, MixerForm.VScrollBarJoyX.Value)
        setScrollbarValue(ComboBoxJoyY.SelectedIndex + 1, MixerForm.VScrollBarJoyY.Value)
        setScrollbarValue(ComboBoxJoyZ.SelectedIndex + 1, MixerForm.VScrollBarJoyZ.Value)
        setScrollbarValue(ComboBoxJoyRx.SelectedIndex + 1, MixerForm.VScrollBarJoyRx.Value)
        setScrollbarValue(ComboBoxJoyRy.SelectedIndex + 1, MixerForm.VScrollBarJoyRy.Value)
        setScrollbarValue(ComboBoxJoyRz.SelectedIndex + 1, MixerForm.VScrollBarJoyRz.Value)
        setScrollbarValue(ComboBoxJoyE1.SelectedIndex + 1, MixerForm.VScrollBarJoyEx1.Value)
        setScrollbarValue(ComboBoxJoyE2.SelectedIndex + 1, MixerForm.VScrollBarJoyEx2.Value)
    End Sub

    Sub setScrollbarValue(ByVal scrollbarNo As Integer, ByVal value As Integer)
        Select Case scrollbarNo
            Case 1
                VScrollBar1.Value = value
            Case 2
                VScrollBar2.Value = value
            Case 3
                VScrollBar3.Value = value
            Case 4
                VScrollBar4.Value = value
            Case 5
                VScrollBar5.Value = value
            Case 6
                VScrollBar6.Value = value
            Case 7
                VScrollBar7.Value = value
            Case 8
                VScrollBar8.Value = value

        End Select
    End Sub

    ''' <summary>
    ''' Called automatically at termination of program, closes COM-Ports
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If Me.SerialPort1.IsOpen Then
            Me.SerialPort1.Close()
        End If
    End Sub

    ''' <summary>
    ''' Button for setting up COM-Port pressed
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnOpenCom_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOpenCom.Click
        If SerialPort1.IsOpen Then
            SerialPort1.Close()
        End If

        Try
            With SerialPort1
                .PortName = Me.ComboBox1.SelectedItem
                .BaudRate = CInt(txtBaudrate.Text)
                .Parity = IO.Ports.Parity.None
                .DataBits = 8
                .StopBits = IO.Ports.StopBits.One
            End With
            SerialPort1.Open()
        Catch ex As Exception
            MsgBox(ex.ToString)
        Finally
            btnCloseCom.Enabled = SerialPort1.IsOpen
            btnOpenCom.Enabled = Not SerialPort1.IsOpen
        End Try
    End Sub

    ''' <summary>
    ''' Button for closing COM-Port pressed
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnCloseCom_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCloseCom.Click
        SerialPort1.Close()
        btnCloseCom.Enabled = SerialPort1.IsOpen
        btnOpenCom.Enabled = Not SerialPort1.IsOpen
    End Sub

    ''' <summary>
    ''' Returns a hexadecimal string that corresponds to the value of the given parameter
    ''' </summary>
    ''' <param name="value"></param>
    ''' <returns>Hex representation of value</returns>
    ''' <remarks></remarks>
    Private Function hexString(ByVal value As Integer) As String
        If value = 0 Then
            Return "0x00"
        Else
            Return "0x" & Hex(value)
        End If
    End Function


    Dim cycleCounter As Integer
    ''' <summary>
    ''' Called periodically to transmit the current channel data via COM-Port
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub TmrSendData_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TmrSendData.Tick
        Dim hexOutput As String                 ' For visualization of output data
        Dim outputChanged As Boolean = False    ' flag; only transmit if data changed

        Try
            Me.TmrSendData.Interval = Me.tbCycleTime.Text
        Catch
        End Try

        If cbJoystickActive.Checked Then
            Me.readJoystickInput()
        End If

        MixerForm.mixerTasks()

        If cbJoystickToPPMChannelActive.Checked Then
            Me.writeMixedJoystickAxes2PPMChannel()
        End If




        If (CheckBox1.Checked = True) Then
            dataOutput(0) = 129
        Else
            dataOutput(0) = 128
        End If
        hexOutput = Me.hexString(dataOutput(0))

        Dim idx, v As Integer

        For idx = 1 To 8
            Select Case idx
                Case 1
                    v = VScrollBar1.Value + VScrollBar1_Offset.Value
                Case 2
                    v = VScrollBar2.Value + VScrollBar2_Offset.Value
                Case 3
                    v = VScrollBar3.Value + VScrollBar3_Offset.Value
                Case 4
                    v = VScrollBar4.Value + VScrollBar4_Offset.Value
                Case 5
                    v = VScrollBar5.Value + VScrollBar5_Offset.Value
                Case 6
                    v = VScrollBar6.Value + VScrollBar6_Offset.Value
                Case 7
                    v = VScrollBar7.Value + VScrollBar7_Offset.Value
                Case 8
                    v = VScrollBar8.Value + VScrollBar8_Offset.Value
            End Select
            v = Math.Min(Math.Max(Me.VScrollBar1.Minimum, v), Me.VScrollBar1.Maximum)
            ' MSB
            If dataOutput(2 * idx - 1) <> v \ 128 Then
                dataOutput(2 * idx - 1) = v \ 128
                outputChanged = True
            End If

            ' LSB
            If dataOutput(2 * idx - 1 + 1) <> v Mod 128 Then
                dataOutput(2 * idx - 1 + 1) = v Mod 128
                outputChanged = True
            End If

            hexOutput &= " " + Me.hexString(dataOutput(2 * idx - 1))
            hexOutput &= " " + Me.hexString(dataOutput(2 * idx - 1 + 1))
        Next idx

        txtTx.Text = hexOutput

        If (Me.btnCloseCom.Enabled = True) And outputChanged Then
            Me.SerialPort1.Write(dataOutput, 0, dataOutput.Length)
            cycleCounter += 1
        End If
    End Sub

    Private Sub VScrollBar8_Offset_Scroll(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles VScrollBar1.Scroll, VScrollBar2.Scroll, VScrollBar3.Scroll, VScrollBar4.Scroll, VScrollBar5.Scroll, VScrollBar6.Scroll, VScrollBar7.Scroll, VScrollBar8.Scroll, VScrollBar1_Offset.Scroll, VScrollBar2_Offset.Scroll, VScrollBar3_Offset.Scroll, VScrollBar4_Offset.Scroll, VScrollBar5_Offset.Scroll, VScrollBar6_Offset.Scroll, VScrollBar7_Offset.Scroll, VScrollBar8_Offset.Scroll, VScrollBarJoyX.Scroll, VScrollBarJoyZ.Scroll, VScrollBarJoyY.Scroll, VScrollBarJoyRz.Scroll, VScrollBarJoyRy.Scroll, VScrollBarJoyRx.Scroll, VScrollBarJoyEx1.Scroll, VScrollBarJoyEx2.Scroll
        TextBox1.Text = e.NewValue
    End Sub

    Private Sub Initialize_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Initialize.Click
        Me.initJoystick()
    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        MixerForm.Visible = True
    End Sub

    Private Sub tbCycleTime_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tbCycleTime.TextChanged
        Dim txt As String = CType(sender, TextBox).Text

        Try
            Dim i As Integer = CInt(txt)
            If i < 10 Then
                CType(sender, TextBox).Text = 10
            End If
        Catch ex As Exception
            CType(sender, TextBox).Text = 50
        End Try
    End Sub
End Class
